@@ -11,6 +11,7 @@ from robotframework_ls.impl.protocols import (
     IRobotWorkspace,
     IKeywordDefinition,
     ILibraryImportNode,
+    KeywordUsageInfo,
 )
 
 
@@ -355,8 +356,10 @@ class CompletionContext(object):
                     # It's a relative resource, resolve its location based on the
                     # current file.
                     check_paths = [
-                        os.path.join(
-                            os.path.dirname(self.doc.path), name_with_resolved_vars
+                        os.path.normpath(
+                            os.path.join(
+                                os.path.dirname(self.doc.path), name_with_resolved_vars
+                            )
                         )
                     ]
                     config = self.config
@@ -365,8 +368,11 @@ class CompletionContext(object):
                             OPTION_ROBOT_PYTHONPATH, list, []
                         ):
                             check_paths.append(
-                                os.path.join(
-                                    additional_pythonpath_entry, name_with_resolved_vars
+                                os.path.normpath(
+                                    os.path.join(
+                                        additional_pythonpath_entry,
+                                        name_with_resolved_vars,
+                                    )
                                 )
                             )
 
@@ -401,25 +407,54 @@ class CompletionContext(object):
 
         return tuple(ret)
 
+    def _resolve_builtin(self, var_name, value_if_not_found, log_info):
+        from robotframework_ls.impl.robot_constants import BUILTIN_VARIABLES_RESOLVED
+
+        ret = BUILTIN_VARIABLES_RESOLVED.get(var_name, Sentinel.SENTINEL)
+        if ret is Sentinel.SENTINEL:
+            log.info(*log_info)
+            return value_if_not_found
+        return ret
+
     def convert_robot_variable(self, var_name, value_if_not_found):
         from robotframework_ls.impl.robot_lsp_constants import OPTION_ROBOT_VARIABLES
 
         if self.config is None:
-            log.info(
-                "Config not available while trying to convert variable: %s", var_name
+            value = self._resolve_builtin(
+                var_name,
+                value_if_not_found,
+                ("Config not available while trying to convert variable: %s", var_name),
             )
-            value = value_if_not_found
         else:
             robot_variables = self.config.get_setting(OPTION_ROBOT_VARIABLES, dict, {})
-            value = robot_variables.get(var_name)
-            if value is None:
-                log.info("Unable to find variable: %s", var_name)
-                value = value_if_not_found
+            value = robot_variables.get(var_name, Sentinel.SENTINEL)
+            if value is Sentinel.SENTINEL:
+                value = self._resolve_builtin(
+                    var_name,
+                    value_if_not_found,
+                    ("Unable to find variable: %s", var_name),
+                )
+
         value = str(value)
         return value
 
     @instance_cache
     def get_current_keyword_definition(self) -> Optional[IKeywordDefinition]:
+        """
+        Provides the current keyword even if we're in its arguments and not actually
+        on the keyword itself.
+        """
+        current_keyword_definition_and_usage_info = (
+            self.get_current_keyword_definition_and_usage_info()
+        )
+        if not current_keyword_definition_and_usage_info:
+            return None
+        return current_keyword_definition_and_usage_info[0]
+
+    @instance_cache
+    def get_current_keyword_definition_and_usage_info(
+        self
+    ) -> Optional[Tuple[IKeywordDefinition, KeywordUsageInfo]]:
         """
         Provides the current keyword even if we're in its arguments and not actually
         on the keyword itself.
@@ -453,7 +488,7 @@ class CompletionContext(object):
                 )
                 if definitions and len(definitions) >= 1:
                     definition: IKeywordDefinition = next(iter(definitions))
-                    return definition
+                    return definition, usage_info
         return None
 
     def __typecheckself__(self) -> None:
